@@ -16,8 +16,7 @@
 
 # This is the preferences dialog.
 
-from gi.repository import Gtk, Gdk, GObject, GdkPixbuf # pylint: disable=E0611
-import io
+from gi.repository import Gtk, Gdk, GObject, GdkPixbuf  # pylint: disable=E0611
 import stat
 
 import threading
@@ -35,7 +34,7 @@ from variety.AddRedditDialog import AddRedditDialog
 from variety.EditFavoriteOperationsDialog import EditFavoriteOperationsDialog
 from variety.AddWallhavenDialog import AddWallhavenDialog
 
-from variety import _, _u
+from variety import _
 
 import os
 import logging
@@ -89,9 +88,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
         self.copyto_chooser = FolderChooser(self.ui.copyto_folder_chooser, self.on_copyto_changed)
         self.slideshow_custom_chooser = FolderChooser(self.ui.slideshow_custom_chooser, self.delayed_apply)
 
-        try:
-            from varietyslideshow import varietyslideshow
-        except:
+        if not Util.check_variety_slideshow_present():
             self.ui.notebook.remove_page(2)
 
         self.reload()
@@ -135,7 +132,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
             self.options = Options()
             self.options.read()
 
-            self.ui.autostart.set_active(os.path.isfile(os.path.expanduser(u"~/.config/autostart/variety.desktop")))
+            self.ui.autostart.set_active(os.path.isfile(os.path.expanduser("~/.config/autostart/variety.desktop")))
 
             self.ui.change_enabled.set_active(self.options.change_enabled)
             self.set_change_interval(self.options.change_interval)
@@ -180,8 +177,6 @@ class PreferencesVarietyDialog(PreferencesDialog):
 
             self.favorites_operations = self.options.favorites_operations
 
-            self.ui.facebook_show_dialog.set_active(self.options.facebook_show_dialog)
-
             self.ui.copyto_enabled.set_active(self.options.copyto_enabled)
             self.copyto_chooser.set_folder(self.parent.get_actual_copyto_folder())
 
@@ -280,7 +275,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
             #self.ui.sources.get_selection().connect("changed", self.on_sources_selection_changed)
 
             if hasattr(self, "filter_checkboxes"):
-                for cb in self.filter_checkboxes:
+                for cb in self.filter_checkboxes:  # pylint: disable=access-member-before-definition
                     self.ui.filters_grid.remove(cb)
                     cb.destroy()
             self.filter_checkboxes = []
@@ -295,6 +290,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
                 self.ui.filters_grid.attach(cb, i % 4, i // 4, 1, 1)
                 self.filter_checkboxes.append(cb)
 
+            # pylint: disable=access-member-before-definition
             if hasattr(self, "quotes_sources_checkboxes"):
                 for cb in self.quotes_sources_checkboxes:
                     self.ui.quotes_sources_grid.remove(cb)
@@ -313,7 +309,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
             self.ui.tips_buffer.set_text('\n\n'.join(Texts.TIPS))
 
             try:
-                with io.open(get_data_file("ui/changes.txt")) as f:
+                with open(get_data_file("ui/changes.txt")) as f:
                     self.ui.changes_buffer.set_text(f.read())
             except Exception:
                 logger.warning(lambda: "Missing ui/changes.txt file")
@@ -549,7 +545,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
         for i, r in enumerate(self.ui.sources.get_model()):
             if r[1] == Options.type_to_str(type):
                 if type == Options.SourceType.FOLDER:
-                    existing[os.path.normpath(_u(r[2]))] = r, i
+                    existing[os.path.normpath(r[2])] = r, i
                 else:
                     existing[self.model_row_to_source(r)[2]] = r, i
 
@@ -626,7 +622,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
         type = Options.str_to_type(edited_row[1])
 
         if type == Options.SourceType.IMAGE or type == Options.SourceType.FOLDER:
-            subprocess.Popen(["xdg-open", os.path.realpath(_u(edited_row[2]))])
+            subprocess.Popen(["xdg-open", os.path.realpath(edited_row[2])])
         elif type == Options.SourceType.FAVORITES:
             subprocess.Popen(["xdg-open", self.parent.options.favorites_folder])
         elif type == Options.SourceType.FETCHED:
@@ -651,6 +647,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
         selected = set(row.get_indices()[0] for row in rows)
         self.ui.use_button.set_sensitive(selected and enabled != selected)
 
+        # pylint: disable=access-member-before-definition
         if hasattr(self, "previous_selection") and rows == self.previous_selection:
             return
 
@@ -674,6 +671,8 @@ class PreferencesVarietyDialog(PreferencesDialog):
 
         def timer_func():
             self.show_thumbs(list(model[row] for row in rows))
+
+        # pylint: disable=access-member-before-definition
         if hasattr(self, "show_timer") and self.show_timer:
             self.show_timer.cancel()
         self.show_timer = threading.Timer(0.3, timer_func)
@@ -689,7 +688,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
     def model_row_to_source(self, row):
         return [row[0],
                 Options.str_to_type(row[1]),
-                Texts.SOURCES[row[1]][0] if row[1] in Texts.SOURCES else _u(row[2])]
+                Texts.SOURCES[row[1]][0] if row[1] in Texts.SOURCES else row[2]]
 
     def source_to_model_row(self, s):
         srctype = Options.type_to_str(s[1])
@@ -713,7 +712,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
                 type = Options.str_to_type(row[1])
                 if type == Options.SourceType.IMAGE:
                     image_count += 1
-                    images.append(_u(row[2]))
+                    images.append(row[2])
                 else:
                     folder = self.parent.get_folder_of_source(self.model_row_to_source(row))
                     image_count += sum(1 for f in Util.list_files(folders=(folder,), filter_func=Util.is_image, max_files=1, randomize=False))
@@ -791,6 +790,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
             self.delayed_apply_with_interval(1)
 
     def delayed_apply_with_interval(self, interval):
+        # pylint: disable=access-member-before-definition
         if not self.loading:
             if hasattr(self, "apply_timer") and self.apply_timer:
                 self.apply_timer.cancel()
@@ -836,7 +836,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
             self.options.clipboard_enabled = self.ui.clipboard_enabled.get_active()
             self.options.clipboard_use_whitelist = self.ui.clipboard_use_whitelist.get_active()
             buf = self.ui.clipboard_hosts.get_buffer()
-            self.options.clipboard_hosts = Util.split(_u(buf.get_text(buf.get_start_iter(), buf.get_end_iter(), False)))
+            self.options.clipboard_hosts = Util.split(buf.get_text(buf.get_start_iter(), buf.get_end_iter(), False))
 
             if self.ui.icon.get_active() == 0:
                 self.options.icon = "Light"
@@ -847,7 +847,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
             elif self.ui.icon.get_active() == 4:
                 self.options.icon = "None"
             elif self.ui.icon.get_active() == 3:
-                file = _u(self.ui.icon_chooser.get_filename())
+                file = self.ui.icon_chooser.get_filename()
                 if file and os.access(file, os.R_OK):
                     self.options.icon = file
                 else:
@@ -864,8 +864,6 @@ class PreferencesVarietyDialog(PreferencesDialog):
                 pass
 
 
-            self.options.facebook_show_dialog = self.ui.facebook_show_dialog.get_active()
-
             self.options.copyto_enabled = self.ui.copyto_enabled.get_active()
             copyto = os.path.normpath(self.copyto_chooser.get_folder())
             if copyto == os.path.normpath(self.parent.get_actual_copyto_folder('Default')):
@@ -896,19 +894,19 @@ class PreferencesVarietyDialog(PreferencesDialog):
                 pass
 
             self.options.clock_enabled = self.ui.clock_enabled.get_active()
-            self.options.clock_font = _u(self.ui.clock_font.get_font_name())
-            self.options.clock_date_font = _u(self.ui.clock_date_font.get_font_name())
+            self.options.clock_font = self.ui.clock_font.get_font_name()
+            self.options.clock_date_font = self.ui.clock_date_font.get_font_name()
 
             self.options.quotes_enabled = self.ui.quotes_enabled.get_active()
-            self.options.quotes_font = _u(self.ui.quotes_font.get_font_name())
+            self.options.quotes_font = self.ui.quotes_font.get_font_name()
             c = self.ui.quotes_text_color.get_color()
             self.options.quotes_text_color = (c.red // 256, c.green // 256, c.blue // 256)
             c = self.ui.quotes_bg_color.get_color()
             self.options.quotes_bg_color = (c.red // 256, c.green // 256, c.blue // 256)
             self.options.quotes_bg_opacity = max(0, min(100, int(self.ui.quotes_bg_opacity.get_value())))
             self.options.quotes_text_shadow = self.ui.quotes_text_shadow.get_active()
-            self.options.quotes_tags = _u(self.ui.quotes_tags.get_text())
-            self.options.quotes_authors = _u(self.ui.quotes_authors.get_text())
+            self.options.quotes_tags = self.ui.quotes_tags.get_text()
+            self.options.quotes_authors = self.ui.quotes_authors.get_text()
             self.options.quotes_change_enabled = self.ui.quotes_change_enabled.get_active()
             self.options.quotes_change_interval = self.get_quotes_change_interval()
             self.options.quotes_width = max(0, min(100, int(self.ui.quotes_width.get_value())))
@@ -978,7 +976,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
                 dialog.destroy()
 
     def update_autostart(self):
-        file = os.path.expanduser(u"~/.config/autostart/variety.desktop")
+        file = os.path.expanduser("~/.config/autostart/variety.desktop")
 
         if not self.ui.autostart.get_active():
             try:
@@ -1086,7 +1084,7 @@ class PreferencesVarietyDialog(PreferencesDialog):
         buf = self.dialog.ui.textbuffer
         buf.set_text('\n'.join(':'.join(x) for x in self.favorites_operations))
         if self.dialog.run() == Gtk.ResponseType.OK:
-            text = _u(buf.get_text(buf.get_start_iter(), buf.get_end_iter(), False))
+            text = buf.get_text(buf.get_start_iter(), buf.get_end_iter(), False)
             self.favorites_operations = list([x.strip().split(':') for x in text.split('\n') if x])
             self.delayed_apply()
         self.dialog.destroy()
