@@ -17,7 +17,6 @@ import bs4
 import requests
 from functools import wraps
 import hashlib
-import io
 import json
 import re
 import os
@@ -26,24 +25,20 @@ import logging
 import string
 import threading
 import time
-import urllib
-import functools
+import urllib.parse
 import datetime
-from urlparse import urlparse
-from PIL import Image
+import sys
+import functools
+import subprocess
+import codecs
 
-from DominantColors import DominantColors
+from PIL import Image
 
 import gi
 gi.require_version('GExiv2', '0.10')
 gi.require_version('PangoCairo', '1.0')
 
-from gi.repository import Gdk, Pango, GdkPixbuf, GLib, GExiv2
-import inspect
-import subprocess
-import platform
-from variety import _u, _str
-
+from gi.repository import Gdk, Pango, GdkPixbuf, GLib, GExiv2, Gio
 
 VARIETY_INFO = "-"
 
@@ -65,25 +60,6 @@ SOURCE_NAME_TO_TYPE = {
 random.seed()
 logger = logging.getLogger('variety')
 
-class LogMethodCalls(object):
-    def __init__(self, func, level):
-        self.level = level
-        self.func = func
-
-    def __get__(self, obj, cls=None):
-        def logcall(*func_args, **func_kwargs):
-            logger.log(self.level, (cls.__name__ if cls else '')+ ": " + self.func.func_name +
-                         '(' + ', '.join(map(_str, func_args)) +
-                         ((', %s' % func_kwargs) if func_kwargs else '') + ')')
-            if inspect.isfunction(self.func) or inspect.isclass(self.func.im_self):
-                ret = self.func(*func_args, **func_kwargs)
-            else:
-                ret = self.func(obj, *func_args, **func_kwargs)
-            return ret
-        for attr in "__module__", "__name__", "__doc__":
-            setattr(logcall, attr, getattr(self.func, attr))
-        return logcall
-
 
 def debounce(seconds):
     """ Decorator that will postpone a functions execution until after wait seconds
@@ -154,7 +130,7 @@ def cache(ttl_seconds=100*365*24*3600, debug=False):
     def decorate(f):
         _cache = {}
 
-        @functools.wraps(f)
+        @wraps(f)
         def decorated(*args):
             cached = _cache.get(args)
             if not cached or cached['timestamp'] < datetime.datetime.now() - datetime.timedelta(seconds=ttl_seconds):
@@ -199,7 +175,7 @@ class VarietyMetadata(GExiv2.Metadata):
             else:
                 return self.get_tag_string(key)
         else:
-            raise KeyError(u'%s: Unknown tag' % key)
+            raise KeyError('%s: Unknown tag' % key)
 
     def __setitem__(self, key, value):
         if key in self.MULTIPLES:
@@ -209,16 +185,113 @@ class VarietyMetadata(GExiv2.Metadata):
         else:
             self.set_tag_string(key, value)
 
+class ModuleProfiler():
+    # How deep in other modules' code we should profile
+    MAX_NONTARGET_DEPTH = 1
+
+    def __init__(self):
+        """
+        Initializes the module profiler.
+        """
+        self.target_paths = []
+
+        # Track how far deep we are in functions outside our target packages
+        # The intent is to only log the first call to outside methods without following them further
+        self.nontarget_depths = {}
+
+    def log_class(self, cls):
+        """
+        Adds the given class' module to the list of modules to be profiled.
+        """
+        modulename = cls.__module__
+        if modulename not in sys.modules:
+            logger.error("ModuleProfiler: Could not add module %r (class %s) to the list of modules to trace - "
+                         "has it been imported entirely?", modulename, cls)
+            return
+
+        module = sys.modules[modulename]
+
+        self.log_module(module, request=cls)
+
+    def log_module(self, module, request=None):
+        """
+        Adds the given module to the list of modules to be profiled.
+        """
+        self.log_path(module.__file__, request=request)
+
+    def log_path(self, path, request=None):
+        """
+        Adds the given module path to the list of profile targets.
+        """
+        self.target_paths.append(path)
+
+        logger.info("ModuleProfiler: added path %s to list of profile targets (request=%s)",
+                    path, request)
+
+    @functools.lru_cache(maxsize=2048)
+    def is_target_path(self, path):
+        """
+        Returns whether the given path matches one of our modules to be profiled.
+        """
+        for target in self.target_paths:
+            if os.path.isdir(target) and path.startswith(target+os.path.sep):
+                return True
+            elif path == target:
+                return True
+        return False
+
+    def start(self):
+        """
+        Starts the module profiler for all future threads.
+        """
+        threading.setprofile(self.profiler)
+
+    def stop(self):
+        """
+        Removes the module profiler globally and from future threads.
+        """
+        if sys.getprofile() != self.profiler:
+            logger.warning("ModuleProfiler: The currently enabled profile function was not ours - unbinding anyways")
+        threading.setprofile(None)
+        sys.setprofile(None)
+
+    def profiler(self, frame, event, arg):
+        filename = frame.f_code.co_filename
+
+        tid = threading.get_ident()
+
+        if not self.is_target_path(filename):
+            if tid not in self.nontarget_depths:
+                # Pick up where the main thread left off
+                self.nontarget_depths[tid] = self.nontarget_depths.get(threading.main_thread().ident, 1)
+            else:
+                self.nontarget_depths[tid] += 1
+        else:
+            self.nontarget_depths[tid] = 0
+
+        tname = threading.current_thread().name
+
+        if event == 'call':
+            if self.nontarget_depths[tid] > self.MAX_NONTARGET_DEPTH:
+                # Don't log past our max depth for packages that we're not tracking
+                return
+            else:
+                # In order: function name, line number, filename
+                s = '[%s] -> Entering function: %s\t(line %s in %s)' % \
+                   (tname, frame.f_code.co_name, frame.f_lineno, filename)
+                if self.nontarget_depths[tid] == self.MAX_NONTARGET_DEPTH:
+                    s += " - not tracing further because MAX_NONTARGET_DEPTH=%s" % self.MAX_NONTARGET_DEPTH
+                logger.debug(s)
+
+
+        elif event == 'return':
+            if self.nontarget_depths[tid] > self.MAX_NONTARGET_DEPTH:
+                return
+
+            logger.debug('[%s] -> Leaving function:  %s\t(line %s in %s)' %
+                         (tname, frame.f_code.co_name, frame.f_lineno, filename))
 
 class Util:
-    @staticmethod
-    def log_all(cls, level=logging.DEBUG):
-        if logger.isEnabledFor(level):
-            for name, meth in inspect.getmembers(cls):
-                if inspect.ismethod(meth) or inspect.isfunction(meth):
-                    setattr(cls, name, LogMethodCalls(meth, level))
-        return cls
-
     @staticmethod
     def sanitize_filename(filename):
         valid_chars = " ,.!-+@()_%s%s" % (string.ascii_letters, string.digits)
@@ -234,7 +307,7 @@ class Util:
         if index > 0:
             filename = filename[:index]
 
-        filename = urllib.unquote_plus(filename)
+        filename = urllib.parse.unquote_plus(filename)
 
         filename = Util.sanitize_filename(filename)
 
@@ -290,6 +363,7 @@ class Util:
     def list_files(files=(), folders=(), filter_func=(lambda f: True), max_files=10000, randomize=True):
         count = 0
         for filepath in files:
+            logger.debug(lambda: 'checking file %s against filter_func %s' % (filepath, filter_func))
             if filter_func(filepath) and os.access(filepath, os.R_OK):
                 count += 1
                 yield filepath
@@ -306,56 +380,32 @@ class Util:
                             random.shuffle(files)
                             random.shuffle(subFolders)
                         for filename in files:
-                            if filter_func(filename):
+                            logger.debug(lambda: 'checking file %s against filter_func %s (root=%s)' % (filename, filter_func, root))
+                            path = os.path.join(root, filename)
+                            if filter_func(path):
                                 count += 1
                                 if count > max_files:
                                     logger.info(lambda: "More than %d files in the folders, stop listing" % max_files)
                                     return
-                                yield os.path.join(root, filename)
+                                yield path
                 except Exception:
-                    logger.exception(lambda: "Cold not walk folder " + folder)
+                    logger.exception(lambda: "Could not walk folder " + folder)
 
     @staticmethod
     def start_force_exit_thread(delay):
         def force_exit():
             time.sleep(delay)
-            print "Exiting takes too long. Calling os.kill."
+            print("Exiting takes too long. Calling os.kill.")
             os.kill(os.getpid(), 9)
         force_exit_thread = threading.Thread(target=force_exit)
         force_exit_thread.daemon = True
         force_exit_thread.start()
 
-    @staticmethod
-    def check_and_update_metadata(filename):
-        if not Util.is_image(filename):
-            # Skip working on stuff that isn't a picture; part of https://bugs.debian.org/893759
-            return False
-
-        try:
-            m = VarietyMetadata(filename)
-        except Exception:
-            logger.exception(lambda: "Could not read metadata for %s" % filename)
-            return False
-
-        try:
-            if 'Xmp.variety.info' in m or 'vrty.org' in m.get('imageURL', ''):
-                logger.info(lambda: 'Updating metadata for %s' % filename)
-                del m["Xmp.variety.info"]
-                if 'imageURL' in m and 'vrty.org' in m['imageURL']:
-                    del m['imageURL']
-                m.write(preserve_timestamps=True)
-
-                return True
-        except Exception:
-            logger.exception(lambda: "Could not read metadata for %s" % filename)
-
-        return False
-
     @staticmethod
     def write_metadata(filename, info):
         try:
             m = VarietyMetadata(filename)
-            for k, v in info.items():
+            for k, v in sorted(info.items()):
                 if k == 'author':
                     m["Xmp.variety." + k] = v
                     if not 'Xmp.dc.creator' in m:
@@ -375,19 +425,19 @@ class Util:
                 elif k == 'sfwRating':
                     m["Xmp.variety." + k] = int(v)
                 elif k == 'extraData':
-                    m["Xmp.variety." + k] = json.dumps(v)
+                    m["Xmp.variety." + k] = json.dumps(v, sort_keys=True)
                 else:
                     m["Xmp.variety." + k] = v
             m.save_file()
             return True
-        except Exception, ex:
+        except Exception as ex:
             # could not write metadata inside file, use json instead
             logger.exception(lambda: "Could not write metadata directly in file, trying json metadata: " + filename)
             try:
-                with io.open(filename + '.metadata.json', 'w', encoding='utf8') as f:
-                    f.write(json.dumps(info, indent=4, ensure_ascii=False, encoding='utf8'))
+                with open(filename + '.metadata.json', 'w', encoding='utf8') as f:
+                    f.write(json.dumps(info, indent=4, ensure_ascii=False, sort_keys=True))
                     return True
-            except Exception, e:
+            except Exception as e:
                 logger.exception(lambda: "Could not write metadata for file " + filename)
                 return False
 
@@ -399,7 +449,7 @@ class Util:
             info = {}
             for k in ["sourceName", "sourceLocation", "sourceURL", "sourceType", "imageURL", "author", "authorURL"]:
                 if "Xmp.variety." + k in m:
-                    info[k] = _u(m["Xmp.variety." + k])
+                    info[k] = m["Xmp.variety." + k]
 
             try:
                 info['sfwRating'] = int(m['Xmp.variety.sfwRating'])
@@ -407,17 +457,17 @@ class Util:
                 pass
 
             try:
-                info['author'] = _u(m['Xmp.dc.creator'][0])
+                info['author'] = m['Xmp.dc.creator'][0]
             except:
                 pass
 
             try:
-                info['headline'] = _u(m['Iptc.Application2.Headline'][0])
+                info['headline'] = m['Iptc.Application2.Headline'][0]
             except:
                 pass
 
             try:
-                info['description'] = _u(m.get_comment())
+                info['description'] = m.get_comment()
             except:
                 pass
 
@@ -427,19 +477,19 @@ class Util:
                 pass
 
             try:
-                info['keywords'] = map(_u, m['Iptc.Application2.Keywords'])
+                info['keywords'] = m['Iptc.Application2.Keywords']
             except:
                 try:
-                    info['keywords'] = map(_u, m['Xmp.dc.subject'])
+                    info['keywords'] = m['Xmp.dc.subject']
                 except:
                     pass
 
             return info
 
-        except Exception, e:
+        except Exception as e:
             # could not read metadata inside file, try reading json metadata instead
             try:
-                with io.open(filename + '.metadata.json', encoding='utf8') as f:
+                with open(filename + '.metadata.json', encoding='utf8') as f:
                     return json.loads(f.read())
 
             except Exception:
@@ -455,14 +505,14 @@ class Util:
         if rating is None:
             for key in ["Xmp.xmp.Rating", "Exif.Image.Rating", "Exif.Image.RatingPercent"]:
                 if key in m:
-                    del m[key]
+                    del m[key]  # pylint: disable=unsupported-delete-operation
         else:
             m["Xmp.xmp.Rating"] = rating
             m["Exif.Image.Rating"] = max(0, rating)
             if rating >= 1:
                 m["Exif.Image.RatingPercent"] = (rating - 1) * 25
             elif "Exif.Image.RatingPercent" in m:
-                del m["Exif.Image.RatingPercent"]
+                del m["Exif.Image.RatingPercent"]  # pylint: disable=unsupported-delete-operation
 
         m.save_file()
 
@@ -503,7 +553,7 @@ class Util:
         return f
 
     @staticmethod
-    def request(url, data=None, stream=False, method=None):
+    def request(url, data=None, stream=False, method=None, timeout=5):
         if url.startswith('//'):
             url = 'http:' + url
         headers = {
@@ -518,7 +568,7 @@ class Util:
                                  headers=headers,
                                  stream=stream,
                                  allow_redirects=True,
-                                 verify=False)
+                                 timeout=timeout)
             r.raise_for_status()
             return r
         except requests.exceptions.SSLError:
@@ -544,7 +594,7 @@ class Util:
 
     @staticmethod
     def html_soup(url, data=None):
-        return bs4.BeautifulSoup(Util.fetch(url, data))
+        return bs4.BeautifulSoup(Util.fetch(url, data), "lxml")
 
     @staticmethod
     def xml_soup(url, data=None):
@@ -604,7 +654,7 @@ class Util:
     @staticmethod
     def gtk_to_fcmatch_font(gtk_font_name):
         fd = Pango.FontDescription(gtk_font_name)
-        family = _u(fd.get_family())
+        family = fd.get_family()
         size = gtk_font_name[gtk_font_name.rindex(' '):].strip()
         rest = gtk_font_name.replace(family, '').strip().replace(' ', ':')
         return family + ":" + rest, size
@@ -624,35 +674,36 @@ class Util:
 
     @staticmethod
     def compare_versions(v1, v2):
-        def _score(v):
-            a = map(int, v.split('.'))
-            while len(a) < 3:
-                a.append(0)
-            return a[0] * 10**6 + a[1] * 10**3 + a[2]
-        s1 = _score(v1)
-        s2 = _score(v2)
-        return -1 if s1 < s2 else (0 if s1 == s2 else 1)
+        from pkg_resources import parse_version
+        pv1 = parse_version(v1)
+        pv2 = parse_version(v2)
+
+        if pv1 == pv2:
+            return 0
+        else:
+            return -1 if pv1 < pv2 else 1
 
     @staticmethod
     def md5(s):
+        if isinstance(s, str):
+            s = s.encode('utf-8')
         return hashlib.md5(s).hexdigest()
 
     @staticmethod
     def md5file(file):
-        with open(file) as f:
+        with open(file, mode='rb') as f:
             return Util.md5(f.read())
 
     @staticmethod
     def random_hash():
         try:
-            return os.urandom(16).encode('hex')
+            return codecs.encode(os.urandom(16), 'hex_codec').decode('utf8')
         except Exception:
-            return ''.join(random.choice(string.hexdigits) for n in xrange(32))
+            return ''.join(random.choice(string.hexdigits) for _ in range(32)).lower()
 
     @staticmethod
     def get_file_icon_name(path):
         try:
-            from gi.repository import Gio
             f = Gio.File.new_for_path(os.path.normpath(os.path.expanduser(path)))
             query_info = f.query_info("standard::icon", Gio.FileQueryInfoFlags.NONE, None)
             return query_info.get_attribute_object("standard::icon").get_names()[0]
@@ -673,11 +724,11 @@ class Util:
             return pics_folder
         except:
             logger.exception(lambda: "Could not get path to Pictures folder. Defaulting to ~/Pictures.")
-            return os.path.expanduser(u'~/Pictures')
+            return os.path.expanduser('~/Pictures')
 
     @staticmethod
     def superuser_exec(*command_args):
-        logger.warning(lambda: "Executing as superuser: %s" % _str(command_args))
+        logger.warning(lambda: "Executing as superuser: %s" % command_args)
         subprocess.check_call(["pkexec"] + list(command_args))
 
     @staticmethod
@@ -707,7 +758,7 @@ class Util:
             return True
 
         try:
-            host = urlparse(url).netloc
+            host = urllib.parse.urlparse(url).netloc
             if host.startswith('interfacelift.com'):
                 return False
 
@@ -763,10 +814,6 @@ class Util:
         except:
             return None
 
-    @staticmethod
-    def get_os_name():
-        return ' '.join(platform.linux_distribution()[0:2])
-
     # makes the Gtk thread execute the given callback.
     @staticmethod
     def add_mainloop_task(callback, *args):
@@ -787,3 +834,12 @@ class Util:
         daemon_thread.start()
         return daemon_thread
 
+    @staticmethod
+    def check_variety_slideshow_present():
+        try:
+            subprocess.check_call(['which', 'variety-slideshow'],
+                                  stderr=subprocess.PIPE, stdout=subprocess.PIPE)
+            return True
+        except:
+            return False
+
