@@ -18,7 +18,6 @@
 
 from gi.repository import Gtk # pylint: disable=E0611
 import os
-import threading
 
 from variety.Util import Util
 
@@ -42,7 +41,7 @@ except (ValueError, ImportError):
 
 from variety_lib import varietyconfig
 
-from variety import _, _u
+from variety import _
 
 import logging
 logger = logging.getLogger('variety')
@@ -155,12 +154,6 @@ class Indicator:
         self.google_image.connect("activate", window.google_image_search)
         self.image_menu.append(self.google_image)
 
-        self.image_menu.append(Gtk.SeparatorMenuItem.new())
-
-        self.publish_fb = Gtk.MenuItem(_("Share on Facebook"))
-        self.publish_fb.connect("activate", window.publish_on_facebook)
-        self.image_menu.append(self.publish_fb)
-
         self.rating_separator = Gtk.SeparatorMenuItem.new()
         self.image_menu.append(self.rating_separator)
 
@@ -232,11 +225,6 @@ class Indicator:
         self.google_quote_author.connect("activate", window.google_quote_author)
         self.quotes_menu.append(self.google_quote_author)
 
-        self.quote_fb = Gtk.MenuItem(_("Share on Facebook"))
-        self.quote_fb.set_use_underline(True)
-        self.quote_fb.connect("activate", window.publish_quote_on_facebook)
-        self.quotes_menu.append(self.quote_fb)
-
         self.quotes_menu.append(Gtk.SeparatorMenuItem.new())
 
         self.quotes_preferences = Gtk.MenuItem(_("Preferences..."))
@@ -278,18 +266,15 @@ class Indicator:
         self.downloads_handler_id = self.downloads.connect("toggled", window.show_hide_downloads)
         self.menu.append(self.downloads)
 
-        try:
-            from varietyslideshow import varietyslideshow
-        except:
-            logger.warning('Variety Slideshow is not installed. This is an optional extension '
-                           'adding pan-and-zoom slideshows to Variety: see '
-                           'https://github.com/peterlevi/variety-slideshow for details')
-        else:
+        if Util.check_variety_slideshow_present():
             self.menu.append(Gtk.SeparatorMenuItem.new())
-
             self.slideshow = Gtk.MenuItem(_("Start Slideshow"))
             self.slideshow.connect("activate", window.on_start_slideshow)
             self.menu.append(self.slideshow)
+        else:
+            logger.warning('Variety Slideshow is not installed. This is an optional extension '
+                           'adding pan-and-zoom slideshows to Variety: see '
+                           'https://github.com/peterlevi/variety-slideshow for details')
 
         self.menu.append(Gtk.SeparatorMenuItem.new())
 
